import os
import argparse
import asyncio
import logging.config
from configparser import ConfigParser, ExtendedInterpolation

from zope.dottedname.resolve import resolve

import aio.app
import aio.config


@asyncio.coroutine
def start_logging():
    # combine system config with any logging config
    parser = ConfigParser(interpolation=ExtendedInterpolation())
    parser.read_dict(aio.config.replicate_config(aio.app.config))

    if os.path.exists('logging.conf'):
        parser.read_file(open('logging.conf'))

    # filter out the logging sections
    logging_sections = ['handler', 'logger', 'formatter']
    logging_config = aio.config.replicate_config(
        parser,
        test_section=lambda section: (
            any([section.startswith(x) for x in logging_sections])))

    logging_parser = ConfigParser()
    logging_parser.read_dict(logging_config)
    logging.config.fileConfig(logging_parser)


@asyncio.coroutine
def runner(argv, app=None, configfile=None,
           signals=None, config_string=None):
    loop = asyncio.get_event_loop()

    if not app:
        app = resolve("aio.app")

    parser = argparse.ArgumentParser(
        prog="aio",
        description='aio app runner')

    parser.add_argument(
        "-c", nargs="?",
        help="configuration file")

    parsed = parser.parse_known_args(argv)

    if parsed[0].c:
        configfile = parsed[0].c

    import aio.config

    # read default aio.app.config
    config = yield from aio.config.parse_config(
        modules=[aio.app])

    # read user config
    config = yield from aio.config.parse_config(
        config=configfile,
        config_string=config_string,
        parser=config)

    # load up builtins and modules
    app.modules = []

    try:
        _modules = config['aio']['builtin']
        for m in _modules.strip('').split('\n'):
            app.modules.append(resolve(m))
    except KeyError:
        pass

    try:
        _modules = config['aio']['modules']
        for m in _modules.strip('').split('\n'):
            app.modules.append(resolve(m))
    except KeyError:
        pass

    aio.app.modules = tuple(aio.app.modules)

    # read module config
    config = yield from aio.config.parse_config(
        modules=aio.app.modules, parser=config)

    # read user config again
    aio.app.config = yield from aio.config.parse_config(
        config=configfile,
        config_string=config_string,
        parser=config)

    commands = dict(
        app.config['aio:builtin_commands'])

    if "aio:commands" in app.config:
        commands.update(
            dict(app.config['aio:commands']))

    parser.add_argument(
        "command", choices=commands,
        help="command to run")
    parser.add_argument(
        'nargs',
        default=[],
        help=argparse.SUPPRESS,
        nargs="*")

    try:
        parsed_args = parser.parse_args(argv)
    except (SystemExit, IndexError):
        parser.print_help()
        loop.stop()
        return

    from aio import signals as _signals

    yield from start_logging()

    app.signals = signals or _signals.Signals()

    if parsed_args.command in commands:
        try:
            task = resolve(commands[parsed_args.command])
        except Exception as e:
            import traceback
            traceback.print_exc()
            print(e)
            loop.stop()

        yield from task(parsed_args.nargs)

    else:
        parser.print_help()
        loop.stop()



import os
import asyncio

from aio.testing import aiofuturetest
from aio.app.testing import AioAppTestCase
from aio.app.runner import runner
from aio.core.exceptions import MissingConfiguration

test_dir = os.path.dirname(__file__)


class AdditionTestServerProtocol(asyncio.Protocol):

    def connection_made(self, transport):
        self.transport = transport

    def data_received(self, data):
        self.transport.write(
            str(
                sum([
                    int(x.strip()) for x in
                    data.decode("utf-8").split("+")])).encode())
        self.transport.close()


@asyncio.coroutine
def test_addition_server(name, protocol, address, port):
    return (
        yield from asyncio.get_event_loop().create_server(
            AdditionTestServerProtocol,
            address, port))

SERVER_CONFIG_NO_FACTORY_OR_PROTOCOL = """
[aio:commands]
run: aio.app.cmd.cmd_run

[server:test]
port: 7070
address: 127.0.0.1
"""

SERVER_CONFIG_NO_PORT = """
[aio:commands]
run: aio.app.cmd.cmd_run

[server:test]
factory: aio.app.tests.test_run_servers.test_addition_server
"""

SERVER_CONFIG_FACTORY = """
[aio:commands]
run: aio.app.cmd.cmd_run

[server:test]
port: 7070
factory: aio.app.tests.test_run_servers.test_addition_server
"""

SERVER_CONFIG_PROTOCOL = """
[aio:commands]
run: aio.app.cmd.cmd_run

[server:test]
port: 7070
protocol: aio.app.tests.test_run_servers.AdditionTestServerProtocol
"""


class RunCommandServersTestCase(AioAppTestCase):

    def test_run_servers_no_factory_or_protocol(self):

        @aiofuturetest
        def run_server():
            yield from runner(
                ['run'],
                config_string=SERVER_CONFIG_NO_FACTORY_OR_PROTOCOL)

        with self.assertRaises(MissingConfiguration):
            run_server()

    def test_run_servers_no_port(self):

        @aiofuturetest
        def run_server():
            yield from runner(
                ['run'],
                config_string=SERVER_CONFIG_NO_PORT)

        with self.assertRaises(MissingConfiguration):
            run_server()

    @aiofuturetest
    def test_run_servers_factory(self):
        yield from runner(
            ['run'],
            config_string=SERVER_CONFIG_FACTORY)

        @asyncio.coroutine
        def test_cb():
            reader, writer = yield from asyncio.open_connection(
                '127.0.0.1', 7070)
            writer.write(b'2 + 2 + 3')
            yield from writer.drain()
            result = yield from reader.read()
            self.assertEqual(int(result), 7)

        return test_cb

    @aiofuturetest
    def test_run_servers_protocol(self):
        yield from runner(
            ['run'],
            config_string=SERVER_CONFIG_PROTOCOL)

        @asyncio.coroutine
        def test_cb():
            reader, writer = yield from asyncio.open_connection(
                '127.0.0.1', 7070)
            writer.write(b'2 + 2 + 3')
            yield from writer.drain()
            result = yield from reader.read()
            self.assertEqual(int(result), 7)

        return test_cb



import os
import asyncio

import aio.app
from aio.testing import aiofuturetest
from aio.app.testing import AioAppTestCase
from aio.app.runner import runner

test_dir = os.path.dirname(__file__)

SCHEDULER_CONFIG = """
[aio:commands]
run: aio.app.cmd.cmd_run

[schedule:test]
every: 2
func: aio.app.tests._test_scheduler
"""


class RunCommandSchedulersTestCase(AioAppTestCase):

    @aiofuturetest(timeout=5)
    def test_run_schedulers(self):

        class Counter:
            hit_count = 0
        counter = Counter()

        @asyncio.coroutine
        def scheduler(name):
            counter.hit_count += 1

        aio.app.tests._test_scheduler = scheduler

        yield from runner(
            ['run'],
            config_string=SCHEDULER_CONFIG)

        @asyncio.coroutine
        def test_complete():
            # this is called 5 seconds after the server has started
            self.assertTrue(counter.hit_count == 3)

        return test_complete



import sys
import asyncio
import argparse
import os
import doctest
from unittest import (
    TestLoader, TestSuite, TextTestRunner)

from zope.dottedname.resolve import resolve

import aio.app

import logging

log = logging.getLogger("aio.app.testing")


@asyncio.coroutine
def cmd_test(argv):
    loop = asyncio.get_event_loop()

    if argv:
        parser = argparse.ArgumentParser(
            prog="aio test",
            description='run aio tests')
        parser.add_argument(
            "modules",
            nargs="*",
            default=None,
            choices=[m.__name__ for m in aio.app.modules],
            help="module to test")

        try:
            parsed = parser.parse_args(argv)
        except (SystemExit, IndexError):
            loop.stop()

        modules = []
        for module in parsed.modules:
            log.debug("Importing: %s" % module)
            modules.append(resolve(module))
    else:
        modules = aio.app.modules

    aio.app.clear()

    errors = 0

    def setUp(self):
        aio.app.clear()

    def tearDown(self):
        aio.app.clear()

    for module in modules:

        try:
            resolve("%s.tests" % module.__name__)
            suite = TestSuite()
            loader = TestLoader()
            this_dir = "%s/tests" % module.__path__[0]
            readme = "%s/README.rst" % module.__path__[0]
            if os.path.exists(readme):
                suite.addTest(doctest.DocFileSuite(
                    readme,
                    module_relative=False,
                    setUp=setUp,
                    tearDown=tearDown,
                    optionflags=doctest.ELLIPSIS))
            package_tests = loader.discover(start_dir=this_dir)
            suite.addTests(package_tests)

            print('Running tests for %s...' % module.__name__)
            print("------------------------------------------"
                  + "----------------------------")
            result = TextTestRunner(verbosity=2).run(suite)
            aio.app.clear()
            print("")
            if result.failures or result.errors:
                errors += 1
        except ImportError:
            print('No tests for %s' % module.__name__)
        except:
            import traceback
            traceback.print_exc()
            errors += 1

    if errors:
        loop.stop()
        sys.exit(1)
    loop.stop()



import os
import io
from configparser import ConfigParser

import aio.app
from aio.testing import aiotest
from aio.testing.contextmanagers import redirect_all
from aio.app.testing import AioAppTestCase
from aio.app.runner import runner
from aio.signals import Signals

TEST_DIR = os.path.dirname(__file__)

CONFIG = """
[aio:commands]
test: aio.testing.cmd.cmd_test
"""


class RunnerTestCase(AioAppTestCase):

    @aiotest
    def test_runner_no_command(self):
        """
        with no args, runner reads config, but does not setup app
        help msg is printed to stdout
        """
        from aio import app

        with io.StringIO() as o, io.StringIO() as e, redirect_all(o, e):
            yield from runner([], config_string=CONFIG)
            stdout = o.getvalue()

        # print help msg
        self.assertTrue(
            stdout.startswith(
                'usage: aio [-h] [-c [C]] {test,run}\n\naio'))

        # config has been loaded
        self.assertIsInstance(app.config, ConfigParser)

        # no signals have been set up
        self.assertIsNone(getattr(app, 'signals', None))

    @aiotest
    def test_runner_bad_command(self):
        from aio import app

        with io.StringIO() as o, io.StringIO() as e, redirect_all(o, e):
            yield from runner(['BAD'], config_string=CONFIG)
            stdout = o.getvalue()
            stderr = e.getvalue()

        self.assertTrue(
            stderr.endswith(
                "invalid choice: 'BAD' (choose from 'test', 'run')\n"))

        self.assertTrue(
            stdout.startswith(
                'usage: aio [-h] [-c [C]] {test,run}'))

        # config is set up
        self.assertIsInstance(app.config, ConfigParser)

        # signals are not
        self.assertIsNone(app.signals)

        # builtin modules are set up
        self.assertEqual(app.modules, (aio.app, ))

    @aiotest
    def test_runner_app_file_conf(self):
        from aio import app
        yield from runner(
            ['run'], configfile=os.path.join(
                TEST_DIR, "resources", "test-1.conf"))
        self.assertIsInstance(app.config, ConfigParser)
        self.assertIsInstance(app.signals, Signals)

    def tearDown(self):
        from aio import app
        if hasattr(app, "signals"):
            del(app.signals)
        if hasattr(app, "config"):
            del(app.config)
        if hasattr(app, "modules"):
            del(app.modules)



import os
import io
from configparser import ConfigParser

from aio.testing import aiotest
from aio.testing.contextmanagers import redirect_all
from aio.app.testing import AioAppTestCase
from aio.app.runner import runner
from aio.signals import Signals

test_dir = os.path.dirname(__file__)


class RunCommandTestCase(AioAppTestCase):

    @aiotest
    def _test_run_command(self):
        """
        with no args, runner reads config, but does not setup app
        help msg is printed to stdout
        """
        from aio import app
        conf = os.path.join(
            test_dir, "resources", "test-2.conf")

        with io.StringIO() as out, redirect_all(out):
            yield from runner(['run'], configfile=conf)
            stdout = out.getvalue()

        self.assertEqual(stdout, "")

        # config has been loaded
        self.assertIsInstance(app.config, ConfigParser)

        # signals have been added
        self.assertIsInstance(app.signals, Signals)



import unittest

import aio.app


class AioAppTestCase(unittest.TestCase):

    def setUp(self):
        super(AioAppTestCase, self).setUp()
        aio.app.clear()

    def tearDown(self):
        super(AioAppTestCase, self).tearDown()
        aio.app.clear()



import asyncio

from aio.testing import aiotest
from aio.app.testing import AioAppTestCase
from aio.app.runner import runner

LISTENER_CONFIG = """
[aio:commands]
run: aio.app.cmd.cmd_run

[listen:foo]
test-signal: aio.app.tests._test_listener
"""


class RunListenersTestCase(AioAppTestCase):

    @aiotest
    def test_run_listeners(self):
        """
        on run a listener for "test-signal" should be set up from config file
        """
        import aio.app

        class Result:
            message = None
        res = Result()

        @asyncio.coroutine
        def test_listener(signal, resp):
            res.message = "%s received: %s" % (
                signal, resp)

        aio.app.tests._test_listener = test_listener

        yield from runner(
            ['run'],
            config_string=LISTENER_CONFIG)

        yield from aio.app.signals.emit('test-signal', 'BOOM!')

        self.assertEqual(
            res.message,
            'test-signal received: BOOM!')
        del aio.app.tests._test_listener



import os
import sys
import asyncio

from aio.app.runner import runner


def main():
    app_dir = os.path.abspath('apps')
    if app_dir not in sys.path:
        sys.path.append(app_dir)
    loop = asyncio.get_event_loop()
    asyncio.async(
        runner(sys.argv[1:]))
    loop.run_forever()
    loop.close()


if __name__ == "__main__":
    main()



signals = None
config = None
modules = ()
servers = {}


def clear():
    import aio.app
    aio.app.signals = None
    aio.app.config = None
    aio.app.modules = ()

    for server in aio.app.servers.values():
        server.close()

    aio.app.servers = {}



"""
aio.app
"""
import os
import sys
from setuptools import setup, find_packages

version = "0.0.13"

install_requires = [
    "distribute",
    "aio.signals",
    "aio.config",
    "zope.dottedname"]

if sys.version_info < (3, 4):
    install_requires += ['asyncio']

tests_require = install_requires + ['aio.testing>=0.2']


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

long_description = (
    'Detailed documentation\n'
    + '**********************\n'
    + '\n'
    + read("README.rst")
    + '\n')

try:
    long_description += (
        '\n'
        + read("aio", "app", "README.rst")
        + '\n')
except FileNotFoundError:
    pass

setup(
    name='aio.app',
    version=version,
    description="Aio application runner",
    long_description=long_description,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3.4",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
    keywords='',
    author='Ryan Northey',
    author_email='ryan@3ca.org.uk',
    url='http://github.com/phlax/aio.app',
    license='GPL',
    packages=find_packages(),
    namespace_packages=['aio'],
    include_package_data=True,
    zip_safe=False,
    tests_require=tests_require,
    test_suite="aio.app.tests",
    install_requires=install_requires,
    entry_points={
        'console_scripts': [
            'aio = aio.app.management:main']})



import asyncio

from zope.dottedname.resolve import resolve
from aio import app
from aio.core.exceptions import MissingConfiguration

import logging

log = logging.getLogger('aio')


def schedule(name, func, cb, t, exc=None):
    log.info(
        'Scheduler started (%s): %s.%s' % (
            name, func.__module__, func.__name__))
    while True:
        future = asyncio.async(func(name))

        def _cb(res):
            if res.exception():
                if exc:
                    asyncio.async(exc(res.exception()))
            else:
                if cb:
                    asyncio.async(cb(res.result()))

        future.add_done_callback(_cb)
        yield from asyncio.sleep(t)


@asyncio.coroutine
def server_factory(name, protocol, address, port):
    loop = asyncio.get_event_loop()
    return (
        yield from loop.create_server(
            protocol, address, port))


@asyncio.coroutine
def start_server(name, address="127.0.0.1", port=8080,
                 factory=None, protocol=None):

    if not port:
        raise MissingConfiguration(
            "Section [server:%s] must specify port to listen on" % name)

    if not factory and not protocol:
        message = (
            "Section [server:%s] must specify one of factory or "
            + "protocol to start server") % name
        raise MissingConfiguration(message)

    if not factory:
        factory = server_factory

    module = "%s.%s" % (factory.__module__, factory.__name__)

    try:
        res = yield from factory(name, protocol, address, port)
    except Exception as e:
        log.error("Server(%s) failed to start: %s" % (name, module))
        raise e

    app.servers[name] = res
    log.info(
        'Server(%s) %s started on %s:%s' % (
            name,
            module,
            address, port))
    return res


@asyncio.coroutine
def cmd_run(argv):
    from aio.app import config

    # yield from app.signals.emit('aio-starting', None)
    log.debug('aio app starting')

    log.debug('adding event listeners')
    for s in config.sections():
        if s.startswith("listen:"):
            msg = s.split(":")[1].strip()
            section = config[s]
            for signal, handlers in section.items():
                for handler in [h.strip() for h in handlers.split('\n')]:
                    app.signals.listen(signal, resolve(handler))

    log.debug('adding schedulers')
    for s in config.sections():
        if s.startswith("schedule:"):
            msg = s.split(":")[1].strip()
            section = config[s]
            every = section.get("every")
            func = resolve(section.get('func'))

            cb = section.get('cb', None)
            if cb:
                cb = resolve(cb)

            err = section.get('err', None)
            if err:
                err = resolve(err)

            log.debug("Starting scheduler: %s" % msg)
            asyncio.async(
                schedule(msg, func, cb, int(every), err))

    log.debug('adding servers')
    for s in config.sections():
        if s.startswith("server:"):
            name = s.split(":")[1].strip()
            section = config[s]
            factory = section.get('factory', None)
            if factory:
                factory = resolve(factory)
            protocol = section.get('protocol', None)
            if protocol:
                protocol = resolve(protocol)
            address = section.get('address')
            port = section.get('port')

            log.debug("Starting server: %s" % name)

            task = asyncio.async(
                start_server(
                    name, address, port, factory, protocol))

            def _server_started(res):
                if res.exception():
                    loop = asyncio.get_event_loop()
                    loop.stop()
                    log.error(res.exception())
                    raise res.exception()

            task.add_done_callback(_server_started)

    log.info('aio app started')
    yield from app.signals.emit('aio-started', None)
